import logging
from typing import Any

import requests
from django.conf import settings
import json

from main.common import logger
from main.models import Geo_Tokens
from main.utils import get_proxy_conf_dict

logger = logging.getLogger(__name__)

headers = {'User-agent': settings.USER_AGENT}


def check_geo_ip_tokens(main, filed="alias") -> list:
    """Check geotoken and update alias for get logs group"""
    tokens_geo = {}
    for token in Geo_Tokens.objects.using('waf').all():
        tokens_geo[token.waf_id] = token.geo_token
    for i in main:
        if isinstance(i, dict):
            i['geotoken'] = tokens_geo.get(str(i[filed]))
        else:
            i.geotoken = tokens_geo.get(str(getattr(i, filed)))
    return main


def chunck_list_ip(ip_dict: list):
    list_with_token = []
    for i in ip_dict:
        if i['geotoken'] in list_with_token:
            index = list_with_token.index(i['geotoken'])
            list_with_token[index + 1].append(i['ip'])
        else:
            list_with_token.append(i['geotoken'])
            list_with_token.append([i['ip']])

    for i in list_with_token:
        if type(i) == list:
            index = list_with_token.index(i)
            list_with_token[index] = [i[x:x + 100] for x in range(0, len(i), 100)]
    return list_with_token


def get_country_by_ip(ip_dict: list) -> dict:
    """ returns short form: FR, US, KZ etc"""
    try:
        country = {}
        check_geo_ip_tokens(ip_dict)
        list_with_token = chunck_list_ip(ip_dict)
        for i in range(0, len(list_with_token), 2):
            for ip_snip in list_with_token[i + 1]:
                url = f"{settings.URI_GEOIP_BATCH}"
                response = requests.post(url + f"{list_with_token[i]}",
                                         headers={'User-agent': settings.USER_AGENT},
                                         proxies=get_proxy_conf_dict(),
                                         timeout=settings.URI_GEOIP_TIMEOUT,
                                         data=f"{ip_snip}",
                                         )
                if response.status_code == 200:
                    json_value_req = json.loads(response.content)
                    for i in json_value_req:
                        for row in json_value_req[i].split('<br>'):
                            if row == "Private IP":
                                country[i] = "Private IP"
                            else:
                                country[i] = json.loads(row)['cc']
            return country
        else:
            return 'n/a'
    except Exception as e:
        logger.error(f'An error occurred error while receiving GeoIP data:{e}')